import pytest

from robocop import __version__
from robocop.linter.reports.robocop_version_report import RobocopVersionReport


class TestRobocopVersionReport:
    @pytest.mark.parametrize("previous_results", [None, {}, {"generated_with": "3.4.0"}])
    @pytest.mark.parametrize("compare_runs", [True, False])
    def test_version_report(self, compare_runs, previous_results, config, capsys):
        config.linter.compare = compare_runs
        report = RobocopVersionReport(config)
        expected_message = f"\nReport generated by Robocop version: {__version__}"
        if previous_results and compare_runs:
            expected_message += ". Previous results generated by Robocop version: 3.4.0\n"
        else:
            expected_message += "\n"
        report.generate_report(previous_results)
        out, _ = capsys.readouterr()
        assert out == expected_message

    @pytest.mark.parametrize("compare_runs", [True, False])
    def test_persistent_save(self, compare_runs, config):
        config.linter.compare = compare_runs
        report = RobocopVersionReport(config)
        expected = {"generated_with": __version__}
        results = report.persist_result()
        assert results == expected
